module receiver(
    input wire      in,
    input wire      clock, /* baudrate * 7 hertz */
    output reg[7:0] out
);

    reg[7:0]    data;
    reg[2:0]    current_bit_value;
    reg[2:0]    stop_bit_value;
    reg[2:0]    start_bit_value;
    reg[3:0]    state_read; // 1 - 10 = reading given bit
    reg[2:0]    state_bit;

    localparam state_read_nop = 0;
    localparam state_read_start_bit = 1;
    localparam state_read_first_data_bit = 2;
    localparam state_read_stop_bit = 10;
    localparam state_read_last_bit = 10;

    localparam state_bit_first_read = 2;
    localparam state_bit_second_read = 3;
    localparam state_bit_third_read = 4;
    localparam state_bit_calculate = 5;
    localparam state_bit_last_segment = 6;
    
    /* Bit state machine */
    always @(posedge clock)
    begin  
        if (in == 0 && state_read == state_read_nop) /* Begin receiving */
        begin
            state_bit   <= 1;
            state_read  <= 1;
        end
        else if (state_read == state_read_last_bit && state_bit == state_bit_last_segment) /* End reading */
        begin
            state_bit   <= 0;
            state_read  <= 0;
        end        
        else if (state_read != state_read_nop) /* Continue in receiving */
        begin
            state_bit <= (state_bit == 6) ? 0 : state_bit + 1;
            state_read <= (state_bit == 6) ? state_read + 1 : state_read;
        end
        
    end
    
    /* Reading data */
    always @(posedge clock)
    begin
        if (state_read == state_read_nop)
        begin
            data <= data;
            if (start_bit_value < 2 && stop_bit_value > 1) /* Publish data if correct */
            begin
                out <= data;
                data <= 0;
            end
            
            /* Clear previous data */
            start_bit_value <= 0;
            stop_bit_value <= 0;
        end
        if (state_read == state_read_start_bit) /* Read start bit */
        begin
            case (state_bit)
                state_bit_first_read:   start_bit_value <= start_bit_value + in;
                state_bit_second_read:  start_bit_value <= start_bit_value + in;
                state_bit_third_read:   start_bit_value <= start_bit_value + in;
            endcase
        end
        if (state_read == state_read_stop_bit) /* Read stop bit */
        begin
            case (state_bit)
                state_bit_first_read:   stop_bit_value <= stop_bit_value + in;
                state_bit_second_read:  stop_bit_value <= stop_bit_value + in;
                state_bit_third_read:   stop_bit_value <= stop_bit_value + in;
            endcase
        end
        if (1 < state_read && state_read < 10) /* Read data bits */
        begin
            case (state_bit)
                state_bit_first_read:   current_bit_value <= current_bit_value + in;
                state_bit_second_read:  current_bit_value <= current_bit_value + in;
                state_bit_third_read:   current_bit_value <= current_bit_value + in;
                state_bit_calculate:
                begin
                    data <= current_bit_value > 1
                        ? (data | (1 << (state_read - state_read_first_data_bit)))
                        : data;
                    current_bit_value <= 0;
                end
            endcase
        end
    end

endmodule
